/* Customizable desktop links for the Midnight Commander
 *
 * Copyright (C) 1998-1999 The Free Software Foundation
 *
 * Authors: Miguel de Icaza <miguel@nuclecu.unam.mx>
 */

#ifndef GDESKTOP_INIT_H
#define GDESKTOP_INIT_H

void gdesktop_links_init (void);

#endif
